use itertools::Itertools;

pub fn part1(input: &str) -> u32 {
    input.lines()
        .filter(|report| {
            let mut levels = [0i8; 8];
            let mut last = 0usize;

            report.split_ascii_whitespace()
                .enumerate()
                .for_each(|(i, level)| {
                    let mut acc = 0i8;

                    for b in level.as_bytes() {
                        acc = acc * 10 + (b - b'0') as i8;
                        last = i;
                    }

                    levels[i] = acc;
                });

            is_safe(&levels[..=last])
        })
        .count() as u32
}

pub fn part2(input: &str) -> u32 {
    input.lines()
        .filter(|report| {
            let mut levels = [0i8; 8];
            let mut last = 0usize;

            report.split_ascii_whitespace()
                .enumerate()
                .for_each(|(i, level)| {
                    levels[i] = level.as_bytes().iter()
                        .fold(0i8, |acc, b| acc * 10 + (b - b'0') as i8);
                    last = i;
                });

            let levels = &levels[..=last];

            is_safe(levels)
                || (0..levels.len()).any(|i| is_safe_without(levels, i))
        }).count() as u32
}

fn is_safe(levels: &[i8]) -> bool {
    let increasing = levels[1] > levels[0];

    levels.windows(2).all(|two_levels| {
        let old = two_levels[0];
        let new = two_levels[1];
        let abs = new.abs_diff(old);
        ((new > old) == increasing) && abs != 0 && abs < 4
    })
}

fn is_safe_without(levels: &[i8], skip_idx: usize) -> bool {
    let increasing = match skip_idx {
        0 => levels[2] > levels[1],
        1 => levels[2] > levels[0],
        _ => levels[1] > levels[0]
    };

    levels.iter()
        .enumerate()
        .filter(|(idx, _)| *idx != skip_idx)
        .tuple_windows()
        .all(|((_, old), (_, new))| {
            let abs = new.abs_diff(*old);
            ((new > old) == increasing) && abs != 0 && abs < 4
        })
}

#[cfg(test)]
mod test {
    use super::*;
    use crate::default_tests;

    const INPUT: &'static str = "7 6 4 2 1
1 2 7 8 9
9 7 6 2 1
1 3 2 4 5
8 6 4 4 1
1 3 6 7 9";

    #[test]
    fn test_example_1() {
        assert_eq!(part1(INPUT), 2);
    }

    #[test]
    fn test_example_2() {
        assert_eq!(part2(INPUT), 4);
    }

    default_tests!(day02, 606, 644);
}