use crate::utils::LINE_SEPARATOR_LEN;
use itertools::Itertools;

const LINE_LENGTH: usize = 140;

pub fn part1(input: &str) -> u32 {
    let bytes = input.as_bytes();

    let line_count = bytes
        .windows(4)
        .filter(|win| win == b"XMAS" || win == b"SAMX")
        .count();

    let col_count = bytes
        .chunks(LINE_LENGTH + LINE_SEPARATOR_LEN)
        .tuple_windows()
        .map(|(line1, line2, line3, line4)| {
            line1
                .iter()
                .zip(line2.iter())
                .zip(line3.iter())
                .zip(line4.iter())
                .filter(|(((&c1, &c2), &c3), &c4)| {
                    &[c1, c2, c3, c4] == b"XMAS" || &[c1, c2, c3, c4] == b"SAMX"
                })
                .count()
        })
        .sum::<usize>();

    let diag_count = bytes
        .chunks(LINE_LENGTH + LINE_SEPARATOR_LEN)
        .tuple_windows()
        .map(|(line1, line2, line3, line4)| {
            let mut count = 0usize;

            let mut buf_tl_br = [0u8; 4];
            let mut buf_bl_tr = [0u8; 4];
            for i in 0..LINE_LENGTH - 3 {
                buf_tl_br[0] = line1[i];
                buf_tl_br[1] = line2[i + 1];
                buf_tl_br[2] = line3[i + 2];
                buf_tl_br[3] = line4[i + 3];

                buf_bl_tr[0] = line4[i];
                buf_bl_tr[1] = line3[i + 1];
                buf_bl_tr[2] = line2[i + 2];
                buf_bl_tr[3] = line1[i + 3];

                count += (&buf_tl_br == b"XMAS" || &buf_tl_br == b"SAMX") as usize;
                count += (&buf_bl_tr == b"XMAS" || &buf_bl_tr == b"SAMX") as usize;
            }

            count
        })
        .sum::<usize>();

    (line_count + col_count + diag_count) as u32
}

pub fn part2(input: &str) -> u32 {
    input
        .as_bytes()
        .chunks(LINE_LENGTH + LINE_SEPARATOR_LEN)
        .tuple_windows()
        .map(|(line1, line2, line3)| {
            let mut count = 0usize;

            let mut buf_tl_br = [0u8; 2];
            let mut buf_bl_tr = [0u8; 2];
            for i in 0..LINE_LENGTH - 1 {
                if line2[i + 1] != b'A' {
                    continue;
                }

                buf_tl_br[0] = line1[i];
                buf_tl_br[1] = line3[i + 2];

                buf_bl_tr[1] = line1[i + 2];
                buf_bl_tr[0] = line3[i];

                count += ((&buf_tl_br == b"MS" || &buf_tl_br == b"SM")
                    && (&buf_bl_tr == b"MS" || &buf_bl_tr == b"SM"))
                    as usize;
            }

            count
        })
        .sum::<usize>() as u32
}

#[cfg(test)]
mod test {
    use super::*;
    use crate::default_tests;

    default_tests!(day04, 2517, 1960);
}
